package de.fhhannover.inform.trust.ifmapj.channel;

import java.io.IOException;
import java.io.InputStream;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLSocketFactory;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.impl.DefaultHttpClientConnection;
import org.apache.http.message.BasicHttpEntityEnclosingRequest;
import org.apache.http.params.BasicHttpParams;

import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;

/**
 * Implementation of the {@link CommunicationHandler} interface using
 * Apache's httpcomponent-core package only.
 * 
 * @author aw
 *
 */
class ApacheCoreCommunicationHandler extends AbstractCommunicationHandler {
	
	private final BasicHttpParams mBasicHttpParams;
	private DefaultHttpClientConnection mHttpConnection;
	private boolean mResponseGzip;
	private BasicHttpEntityEnclosingRequest mHttpPost;
	
	ApacheCoreCommunicationHandler(String url, String user, String pass,
			SSLSocketFactory sslSocketFactory, HostnameVerifier verifier)
			throws InitializationException {
		super(url, user, pass, sslSocketFactory, verifier);
		
		mBasicHttpParams = new BasicHttpParams();
	}

	@Override
	public InputStream doActualRequest(InputStream is) throws IOException {
		InputStreamEntity ise = null;
		HttpResponse response = null;
		InputStream ret = null;
		Header hdr = null;
		HttpEntity respEntity = null;
		StatusLine status = null;
		
		
		ise = new InputStreamEntity(is, is.available());
		ise.setChunked(false);
		mHttpPost.setEntity(ise);
		
		// do the actual request
		try {
			mHttpConnection.sendRequestHeader(mHttpPost);
			mHttpConnection.sendRequestEntity(mHttpPost);
			response = mHttpConnection.receiveResponseHeader();
			mHttpConnection.receiveResponseEntity(response);
		} catch (HttpException e) {
			throw new IOException(e);
		}

		// check if we got a 200 status back, otherwise go crazy
		status = response.getStatusLine();
		if (status.getStatusCode() != 200) {
			throw new IOException("HTTP Status Code: "
					+ status.getStatusCode() + " " 
					+ status.getReasonPhrase());
		}
		
		// check whether the response uses gzip
		hdr = response.getFirstHeader("Content-Encoding");
		mResponseGzip = (hdr == null) ? false : hdr.getValue().contains("gzip");
		respEntity = response.getEntity();
		
		if (respEntity != null)
			ret = respEntity.getContent();
		
		if (ret == null)
			throw new IOException("no content in response");
		
		return ret;
	}
	
	@Override
	protected void prepareCommunication() throws IOException {
		if (mHttpConnection == null) {
			mHttpConnection = new DefaultHttpClientConnection();
			mHttpConnection.bind(getSocket(), mBasicHttpParams);
		}
	}

	@Override
	protected void createPostRequest(String path) throws IOException {
		mHttpPost = new BasicHttpEntityEnclosingRequest("POST", path);
	}

	@Override
	protected void addHeader(String key, String value) throws IOException {
		mHttpPost.addHeader(key, value);
	}

	@Override
	protected void finishHeaders() throws IOException {
		// noop, we do this in doActualRequest()
	}

	@Override
	protected boolean replyIsGzipped() throws IOException {
		return mResponseGzip;
	}
	
	@Override
	public void closeTcpConnectionImpl() throws IOException {
	
		IOException tmp = null;
		try {
			if (mHttpConnection != null) 
				mHttpConnection.close();
		} catch (IOException e) {
			tmp = e;
		} finally {
			mHttpConnection = null;
		}
		
		if (tmp != null)
			throw tmp;
	}
}
