package de.fhhannover.inform.trust.ifmapj.exception;

import de.fhhannover.inform.trust.ifmapj.channel.ARC;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.messages.PollResult;

/**
 * Exception to be thrown if a MAPS replied with an ErrorResult to any of the
 * methods {@link SSRC} and {@link ARC} offers.
 * 
 * @author aw
 *
 */
public class IfmapErrorResult extends Exception {
	
	/**
	 * auto-generated
	 */
	private static final long serialVersionUID = 5677886243271636884L;

	/**
	 * represents the errorCode used by the MAPS
	 */
	private final IfmapErrorCode mErrorCode;
	
	/**
	 * represents the errorString used by the MAPS
	 */
	private final String mErrorString;

	/**
	 * represents the name of the subscription if contained in errorResults
	 * of a {@link PollResult}
	 */
	private final String mName;
	
	/**
	 * Construct a {@link IfmapErrorResult} exception with obligatory parameters
	 * 
	 * @param errCode
	 * @param errStr
	 * @throws NullPointerException if errStr is null
	 */
	public IfmapErrorResult(IfmapErrorCode errCode, String errStr) {
		this(errCode, errStr, null);
	}
	
	public IfmapErrorResult(IfmapErrorCode errCode, String errStr, String name) {
		
		if (errStr == null)
			throw new NullPointerException("errStr is not allowed to be null");
		
		mErrorCode = errCode;
		mErrorString = errStr;
		mName = name;
	}

	public IfmapErrorCode getErrorCode() {
		return mErrorCode;
	}

	public String getErrorString() {
		return mErrorString;
	}

	public String getName() {
		return mName;
	}
	
	public String toString() {
		return String.format("error{%s, %s%s}",
				getErrorCode(), getErrorString(),
				(mName != null) ? ", name=" + mName : "");
	}
}
