package de.fhhannover.inform.trust.ifmapj.identifier;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.fhhannover.inform.trust.ifmapj.exception.MarshalException;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;

/**
 * Provides generic access to marshal and unmarshal a {@link Identifier}
 * implementation from or to {@link Element} representation.
 * 
 * @author aw
 *
 */
public interface IdentifierHandler<T extends Identifier> {
	
	/**
	 * Create a {@link Element} instance representing the given {@link Identifier}.
	 * 
	 * @param i the {@link Identifier}
	 * @param doc the root {@link Document} where the element is to be added.
	 *            An implementation <b>must not</b> add the element to the given {@link Document}.
	 * @return the {@link Element} representation of the given {@link Identifier}.
	 * @throws MarshalException If some constraints for the {@link Identifier} are not fullfilled.
	 */
	public Element toElement(Identifier i, Document doc) throws MarshalException;
	
	/**
	 * Create a {@link Identifier} instance from the given {@link Element}.
	 * 
	 * @param el the {@link Element} instance
	 * @return an {@link Identifier} instance representation of the given {@link Element},
	 *         or null if this {@link IdentifierHandler} was not able to parse
	 *         the given {@link Element}.
	 * @throws UnmarshalException If some constraint of the {@link Identifier} was
	 *         not fulfilled. <b>Note, if this handler is not responsible for the
	 *         given XML, null has to be returned.</b>
	 */
	public T fromElement(Element el) throws UnmarshalException;
	
	/**
	 * @return the {@link Class} object for the {@link Identifier} implementation
	 *         this implementation is able to handle.
	 */
	public Class<T> handles();
}
