package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.Collection;
import java.util.List;

import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;

/**
 * Interface to access {@link SearchResult} objects contained in a {@link PollResult}.
 * 
 * <b>Note:</b> All returned collections should be assumed to be unmodifiable.
 * 
 * @author aw
 * 
 * FIXME: We have to return the SearchResults in ordered...!!!
 *
 */
public interface PollResult extends Result {
	
	/**
	 * Get all {@link SearchResult} objects that were received after the initial
	 * poll has been sent to the MAPS.
	 * 
	 * @return an unmodifiable collection of {@link SearchResult} objects
	 * 
	 * @deprecated
	 * No ordering enforced. Replace with {@link #getResults()} and 
	 * {@link SearchResult#getType()}.
	 */
	public Collection<SearchResult> getSearchResults();

	/**
	 * Get all {@link SearchResult} objects for new metadata that has been
	 * published to the MAPS since the last poll.
	 * 
	 * @return an unmodifiable collection of {@link SearchResult} objects
	 * 
	 * @deprecated
	 * No ordering enforced. Replace with {@link #getResults()} and 
	 * {@link SearchResult#getType()}.
	 */
	public Collection<SearchResult> getUpdateResults();

	/**
	 * Get all {@link SearchResult} objects for metadata that has been
	 * deleted from the MAPS since the last poll.
	 * 
	 * @return an unmodifiable collection of {@link SearchResult} objects
	 * 
	 * @deprecated
	 * No ordering enforced. Replace with {@link #getResults()} and 
	 * {@link SearchResult#getType()}.
	 */
	public Collection<SearchResult> getDeleteResults();

	/**
	 * Get all {@link SearchResult} objects for metadata that has been
	 * published to the MAPS by using the publish notify request since the last
	 * poll.
	 * 
	 * @return an unmodifiable collection of {@link SearchResult} objects
	 * 
	 * @deprecated
	 * No ordering enforced. Replace with {@link #getResults()} and 
	 * {@link SearchResult#getType()}.
	 */
	public Collection<SearchResult> getNotifyResults();

	/**
	 * Get all {@link IfmapErrorResult} objects that indicate errors that
	 * occured since the last poll.
	 * 
	 * @return an unmodifiable collection of {@link IfmapErrorResult} objects
	 */
	public Collection<IfmapErrorResult> getErrorResults();
	
	/**
	 * @return an unmodifieable list of {@link SearchResult} instances,
	 * ordered the way they were received from the MAPS.
	 */
	public List<SearchResult> getResults();
}
