package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult.Type;

/**
 * Implementation of {@link PollResult}
 * 
 * @author aw
 *
 */
class PollResultImpl implements PollResult {
	
	private final List<SearchResult> mResults;
	private final Collection<IfmapErrorResult> mErrorResults;
	
	PollResultImpl (List<SearchResult> results, Collection<IfmapErrorResult> eres) {
		
		if (results == null || eres == null)
			throw new NullPointerException("result list is null");
			
		mResults = new ArrayList<SearchResult>(results);
		mErrorResults = new ArrayList<IfmapErrorResult>(eres);
	}
	
	@Override
	public List<SearchResult> getResults() {
		return Collections.unmodifiableList(mResults);
	}

	@Override
	public Collection<SearchResult> getSearchResults() {
		return resultsOfType(SearchResult.Type.searchResult);
	}

	@Override
	public Collection<SearchResult> getUpdateResults() {
		return resultsOfType(SearchResult.Type.updateResult);
	}

	@Override
	public Collection<SearchResult> getDeleteResults() {
		return resultsOfType(SearchResult.Type.deleteResult);
	}

	@Override
	public Collection<SearchResult> getNotifyResults() {
		return resultsOfType(SearchResult.Type.notifyResult);
	}

	@Override
	public Collection<IfmapErrorResult> getErrorResults() {
		return Collections.unmodifiableCollection(mErrorResults);
	}

	private Collection<SearchResult> resultsOfType(Type type) {
		List<SearchResult> ret = new ArrayList<SearchResult>();
		
		for (SearchResult sr : mResults)
			if (sr.getType() == type)
				ret.add(sr);
		
		return Collections.unmodifiableCollection(ret);
	}
}
