package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.List;

/**
 * Interface to access data concerning a {@link PublishRequest}.
 * 
 * MAPCs can publish or delete metadata on a MAPS. A {@link PublishRequest}
 * is used to do these operations. A {@link PublishRequest} can hold one or
 * more of {@link PublishUpdate}, {@link PublishNotify}
 * and {@link PublishDelete} elements.
 * 
 * @author aw
 *
 */
public interface PublishRequest extends Request {
	
	/**
	 * Add a new {@link PublishElement} to the request
	 * 
	 * @param pe the element to add to the list
	 * @throws NullPointerException if the given element is null
	 */
	public void addPublishElement(PublishElement pe);
	
	/**
	 * Get all {@link PublishElement} instances of this request
	 * 
	 * @return the list of elements in the request. <b>Note:</b> The returned collection is unmodifiable.
	 */
	public List<PublishElement> getPublishElements();
}
