package de.fhhannover.inform.trust.ifmapj.messages;

import de.fhhannover.inform.trust.ifmapj.channel.ARC;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;

/**
 * Simple interface indicating an IF-MAP request and providing access to
 * the session-id attribute.
 * 
 * A IF-MAP request is any possible element in the SOAP Body element.
 * All requests, with the exception newSession, implement this interface.
 * 
 * Users of ifmapJ should never directly use this interface, as the session-id
 * will be set by the {@link SSRC} or {@link ARC} implementation automatically.
 * 
 * @author aw
 * 
 */
public interface Request {
	
	/**
	 * @param sessionId the session-d to be used for this request.
	 * 
	 * @throws NullPointerException if sessionId is null.
	 */
	public void setSessionId(String sessionId);
	
	/**
	 * @return the session-id for this request, null if not set.
	 */
	public String getSessionId();
}
