package de.fhhannover.inform.trust.ifmapj.messages;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A {@link SearchResult} simply contains a list of {@link ResultItem} objects.
 * Optionally it may contain a name in case this {@link SearchResult} is
 * contained in a {@link PollResult};
 * 
 * @author aw
 */
class SearchResultImpl implements SearchResult {
	
	private final List<ResultItem> mResultItems;
	
	private final String mName;
	
	private final Type mType;

	SearchResultImpl(List<ResultItem> ritems, String name, Type type) {
		
		if (ritems == null)
			throw new NullPointerException("ritems is null");
		
		if (type == null)
			throw new NullPointerException("type is null");
			
		mResultItems = new ArrayList<ResultItem>(ritems);
		mName = name;
		mType = type;
	}
	
	/**
	 * Easy constructor for searchResults
	 * 
	 * @param ritems
	 */
	SearchResultImpl(List<ResultItem> ritems) {
		this(ritems, null, Type.searchResult);
	}

	@Override
	public List<ResultItem> getResultItems() {
		return Collections.unmodifiableList(mResultItems);
	}

	@Override
	public String getName() {
		return mName;
	}

	@Override
	public Type getType() {
		return mType;
	}
	
	@Override
	public String toString() {
		int i = 0;
		StringBuilder sb = new StringBuilder("sr{");
		if (mName != null) {
			sb.append(mName);
			sb.append(", ");
		}
		for (ResultItem ri : mResultItems) {
			i++;
			sb.append(ri.toString());
			if (i != mResultItems.size())
				sb.append(", ");
		}
		sb.append("}");
		return sb.toString();
	}
}
