package de.fhhannover.inform.trust.ifmapj;

import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;

public class IfmapJHelperTest extends TestCase {
	
	@Test
	public void testGetKeyManagersOk() {
		// set necessary system properties
		System.setProperty("javax.net.ssl.keyStore", Config.KEY_STORE_PATH);
		System.setProperty("javax.net.ssl.keyStorePassword", Config.KEY_STORE_PASSWORD);
		try {
			KeyManager[] km = IfmapJHelper.getKeyManagers();
			assertEquals(km.length, 1);
		} catch (InitializationException e) {
			assertFalse("getKeyManagers failed unexpectedly", true);
		}
	}
	
	@Test
	public void testGetKeyManagersFail() {
		System.setProperty("javax.net.ssl.keyStore", Config.KEY_STORE_PATH);
		System.setProperty("javax.net.ssl.keyStorePassword", Config.KEY_STORE_PASSWORD);
		
		System.clearProperty("javax.net.ssl.keyStore");
		try {
			IfmapJHelper.getKeyManagers();
			assertFalse("getKeyManagers did not fail but was expected to do so", true);
		} catch (InitializationException e) {
			assertTrue(true);
		}
		
		System.setProperty("javax.net.ssl.keyStore", Config.KEY_STORE_PATH);
		System.clearProperty("javax.net.ssl.keyStorePassword");
		try {
			IfmapJHelper.getKeyManagers();
			assertFalse("getKeyManagers did not fail but was expected to do so", true);
		} catch (InitializationException e) {
			assertTrue(true);
		}
		
		System.clearProperty("javax.net.ssl.keyStore");
		try {
			IfmapJHelper.getKeyManagers();
			assertFalse("getKeyManagers did not fail but was expected to do so", true);
		} catch (InitializationException e) {
			assertTrue(true);
		}
	}
	
	@Test
	public void testGetTrustManagersOk() {
		// set necessary system properties
		System.setProperty("javax.net.ssl.trustStore", Config.TRUST_STORE_PATH);
		System.setProperty("javax.net.ssl.trustStorePassword", Config.TRUST_STORE_PASSWORD);
		try {
			TrustManager[] tm = IfmapJHelper.getTrustManagers();
			assertEquals(tm.length, 1);
		} catch (InitializationException e) {
			e.printStackTrace();
			assertFalse("getTrustManagers failed unexpectedly", true);
		}
	}
	
	@Test
	public void testGetTrustManagersFail() {
		System.setProperty("javax.net.ssl.trustStore", Config.TRUST_STORE_PATH);
		System.setProperty("javax.net.ssl.trustStorePassword", Config.TRUST_STORE_PASSWORD);
		
		System.clearProperty("javax.net.ssl.trustStore");
		try {
			IfmapJHelper.getTrustManagers();
			assertFalse("getTrustManagers did not fail but was expected to do so", true);
		} catch (InitializationException e) {
			assertTrue(true);
		}
		
		System.setProperty("javax.net.ssl.trustStore", Config.TRUST_STORE_PATH);
		System.clearProperty("javax.net.ssl.trustStorePassword");
		try {
			IfmapJHelper.getTrustManagers();
			assertFalse("getTrustManagers did not fail but was expected to do so", true);
		} catch (InitializationException e) {
			assertTrue(true);
		}
		
		System.clearProperty("javax.net.ssl.trustStore");
		try {
			IfmapJHelper.getTrustManagers();
			assertFalse("getTrustManagers did not fail but was expected to do so", true);
		} catch (InitializationException e) {
			assertTrue(true);
		}
	}
	
}
