package de.fhhannover.inform.trust.ifmapj.identifier;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import javax.xml.parsers.DocumentBuilder;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.fhhannover.inform.trust.ifmapj.exception.MarshalException;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;

import util.DomHelpers;

public class DeviceHandlerTest {
	
	private static DocumentBuilder sDocBuilder = DomHelpers.newDocumentBuilder();
	private static DeviceHandler sDevHandler = new DeviceHandler();
	
	@Test(expected=MarshalException.class)
	public void testToElementWithNullName() throws MarshalException {
		Document doc = sDocBuilder.newDocument();
		Device dev = Identifiers.createDev(null);
		
		Element res = sDevHandler.toElement(dev, doc);
		assertNull(res);
	}

	@Test(expected=MarshalException.class)
	public void testToElementWithEmptyName() throws MarshalException {
		Document doc = sDocBuilder.newDocument();
		Device dev = Identifiers.createDev("");
		
		Element res = sDevHandler.toElement(dev, doc);
		assertNull(res);
	}
		
	@Test
	public void testToElementWithName() throws MarshalException {
		Document doc = sDocBuilder.newDocument();
		Device dev = Identifiers.createDev("DEV100");
		
		Element xmlName = null;
		Element res = sDevHandler.toElement(dev, doc);
		assertNotNull(res);
		assertEquals("device", res.getLocalName());
		assertNull(res.getPrefix());
		assertNull(res.getNamespaceURI());
		assertEquals(1, res.getChildNodes().getLength());
		xmlName = (Element)res.getFirstChild();
		assertNotNull(xmlName);
		assertEquals("name", xmlName.getLocalName());
		assertEquals("DEV100", xmlName.getTextContent());
		
		assertNull(res.getAttributeNode("administrative-domain"));
		
		assertEquals(1, res.getChildNodes().getLength());
		assertEquals(0, res.getAttributes().getLength());
		assertEquals(1, xmlName.getChildNodes().getLength());
		assertEquals(0, xmlName.getAttributes().getLength());
		assertEquals(Node.TEXT_NODE, xmlName.getFirstChild().getNodeType());
	}

	@Test
	public void testFromElementWithName() throws UnmarshalException {
		Document doc = sDocBuilder.newDocument();
		Element xmlDev = doc.createElementNS(null, "device");
		Element xmlName = doc.createElementNS(null, "name");
		xmlDev.appendChild(xmlName);
		xmlName.setTextContent("DEV100");
		
		Device dev = sDevHandler.fromElement(xmlDev);
		assertNotNull(dev);
		assertNotNull(dev.getName());
		assertEquals("DEV100", dev.getName());
	}

	@Test(expected=UnmarshalException.class)
	public void testFromElementWithNoName() throws UnmarshalException {
		Document doc = sDocBuilder.newDocument();
		Element xmlDev = doc.createElementNS(null, "device");
		Device dev = sDevHandler.fromElement(xmlDev);
		assertNull(dev);
	}

	@Test(expected=UnmarshalException.class)
	public void testFromElementWithEmptyName() throws UnmarshalException {
		Document doc = sDocBuilder.newDocument();
		Element xmlDev = doc.createElementNS(null, "device");
		Element xmlName = doc.createElementNS(null, "name");
		xmlName.setTextContent("");
		xmlDev.appendChild(xmlName);
		Device dev = sDevHandler.fromElement(xmlDev);
		assertNull(dev);
	}

	@Test(expected=UnmarshalException.class)
	public void testFromElementWithBadElement() throws UnmarshalException {
		Document doc = sDocBuilder.newDocument();
		Element xmlDev = doc.createElementNS(null, "device");
		Element xmlName = doc.createElementNS(null, "bad_name_element");
		xmlDev.appendChild(xmlName);
		Device dev = sDevHandler.fromElement(xmlDev);
		assertNull(dev);
	}
	
	@Test
	public void testNotResponsible() throws UnmarshalException {
		Document doc = sDocBuilder.newDocument();
		Element xmlAr = doc.createElementNS(null, "ip-address");
		Device dev = sDevHandler.fromElement(xmlAr);
		assertNull(dev);
	}
	
	@Test(expected=MarshalException.class)
	public void testToElementWrongIdentifierType() throws UnmarshalException, MarshalException {
		Document doc = sDocBuilder.newDocument();
		Element ret = sDevHandler.toElement(Identifiers.createAr("ABC"), doc);
		assertNull(ret);
	}
}
