package de.fhhannover.inform.trust.ifmapj.identifier;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import org.junit.Test;

/**
 * Trivial tests for {@link Identity} class.
 * 
 * @author ibente
 *
 */
public class IdentityTest {
	public String ad = "de.fhhannover.inform.trust";
	public String name = "publisherid:number";
	public String otd = "vendorid:definition";

	@Test
	public void testIdentity() {
		Identity id = Identifiers.createIdentity(null, null);
		
		assertNotNull(id);
		assertNull(id.getAdministrativeDomain());
		assertNull(id.getName());
		assertNull(id.getOtherTypeDefinition());
		assertNull(id.getType());
		
		id = Identifiers.createIdentity(IdentityType.userName, name, ad, otd);
		
		assertEquals(ad, id.getAdministrativeDomain());
		assertEquals(name, id.getName());
		assertEquals(otd, id.getOtherTypeDefinition());
		assertEquals(IdentityType.userName, id.getType());
	}

	@Test
	public void testIdentityToString() {
		
		Identity id = Identifiers.createIdentity(null, null);
		
		assertEquals("id{null, null}", id.toString());
		
		id = Identifiers.createIdentity(IdentityType.aikName, null);
		assertEquals("id{null, aik-name}", id.toString());
	
		id = Identifiers.createIdentity(IdentityType.dnsName, null, "abc");
		assertEquals("id{null, dns-name, abc}", id.toString());
		

		id = Identifiers.createIdentity(IdentityType.dnsName,
				"www.google.com", "abc");
		assertEquals("id{www.google.com, dns-name, abc}", id.toString());
		

		id = Identifiers.createIdentity(IdentityType.other,
				"abc", "abc", null);
		assertEquals("id{abc, other, null, abc}", id.toString());
		
		id = Identifiers.createIdentity(IdentityType.other,
				"abc", "", null);
		assertEquals("id{abc, other, null}", id.toString());
		
		id = Identifiers.createIdentity(IdentityType.other,
				"abc", "", "the-other-guy");
		assertEquals("id{abc, other, the-other-guy}", id.toString());
		
		id = Identifiers.createIdentity(IdentityType.other,
				"abc", "the-other-side", "the-other-guy");
		assertEquals("id{abc, other, the-other-guy, the-other-side}",
				id.toString());
	}
}
