package de.fhhannover.inform.trust.ifmapj.identifier;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.Arrays;
import java.util.Collection;

import javax.xml.parsers.DocumentBuilder;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import util.DomHelpers;
import de.fhhannover.inform.trust.ifmapj.exception.MarshalException;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;

@RunWith(Parameterized.class)
public class IdentityTypeTest {
	private static DocumentBuilder sDocBuilder = DomHelpers.newDocumentBuilder();
	private static IdentityHandler sIdentityHandler = new IdentityHandler();

	private final IdentityType mType;
	private final String mString;

	@SuppressWarnings("rawtypes")
	@Parameters
	public static Collection testParams() {
		return Arrays.asList(new Object[][] {
				{IdentityType.aikName, "aik-name"},
				{IdentityType.distinguishedName, "distinguished-name"},
				{IdentityType.dnsName, "dns-name"},
				{IdentityType.emailAddress, "email-address"},
				{IdentityType.hipHit, "hip-hit"},
				{IdentityType.kerberosPrincipal, "kerberos-principal"},
				{IdentityType.userName, "username"},
				{IdentityType.sipUri, "sip-uri"},
				{IdentityType.telUri, "tel-uri"},
				{IdentityType.other, "other"}
		});
	}

	public IdentityTypeTest(IdentityType type, String expectedTypeString) {

		mString = expectedTypeString;
		mType = type;
	}
	

	@Test
	public void fromElementTypeTest() throws UnmarshalException {

		Document doc = sDocBuilder.newDocument();
		Element xmlId = doc.createElementNS(null, "identity");

		xmlId.setAttribute("name", "USER100");
		xmlId.setAttribute("type", mString);

		// otherwise parsing fails
		if (mString.equals("other"))
			xmlId.setAttribute("other-type-definition", "OTHER_TYPE_DEF");

		Identity i = sIdentityHandler.fromElement(xmlId);
		assertNotNull(i);
		assertEquals(mType, i.getType());
	}

	@Test
	public void toElementTypeTest() throws MarshalException {

		Document doc = sDocBuilder.newDocument();
		Identity id;

		if (mType == IdentityType.other)
			id = Identifiers.createOtherIdentity("USER100", null, "OTHER_TYPE_DEF");
		else
			id = Identifiers.createIdentity(mType, "USER100");



		Element el = sIdentityHandler.toElement(id, doc);
		assertNotNull(el);
		assertEquals("identity", el.getLocalName());
		assertEquals(mString, el.getAttribute("type"));
	}
}
