package de.fhhannover.inform.trust.ifmapj.identifier;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import org.junit.Test;

/**
 * Trivial tests for {@link IpAddress} class.
 * 
 * @author ibente
 *
 */
public class IpAddressTest {

	public String ad = "de.fhhannover.inform.trust";
	public String value = "127.0.0.1";

	@Test
	public void testIpAddress() {
		IpAddress ip = Identifiers.createIp(null, null, null);
		
		assertNotNull(ip);
		assertNull(ip.getAdministrativeDomain());
		assertNull(ip.getValue());
		assertNotNull(ip.getType());
		assertEquals(IpAddressType.IPv4, ip.getType());
	
		ip = Identifiers.createIp(IpAddressType.IPv6, value, ad);
		
		assertEquals(ad, ip.getAdministrativeDomain());
		assertEquals(value, ip.getValue());
		assertEquals(IpAddressType.IPv6, ip.getType());
	}

	@Test
	public void testIpAddressToString() {
		
		IpAddress ip = Identifiers.createIp(null, null, null);
		
		assertEquals("ip{null, IPv4}", ip.toString());
		
		ip = Identifiers.createIp(null, null, "abc");
		assertEquals("ip{null, IPv4, abc}", ip.toString());
		
		ip = Identifiers.createIp(IpAddressType.IPv6, null, "abc");
		assertEquals("ip{null, IPv6, abc}", ip.toString());
		
		ip = Identifiers.createIp(IpAddressType.IPv6,
				"i'll never remember how it looks like",
				"abc");
		assertEquals("ip{i'll never remember how it looks like, IPv6, abc}",
				ip.toString());

		/* this actually tests the super class */
		ip = Identifiers.createIp6("i'll never remember how it looks like");
		assertEquals("ip{i'll never remember how it looks like, IPv6}",
				ip.toString());
		
		ip = Identifiers.createIp4("192.168.0.1");
		assertEquals("ip{192.168.0.1, IPv4}", ip.toString());
	}
}
