package de.fhhannover.inform.trust.ifmapj.messages;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Collection;

import javax.xml.parsers.DocumentBuilder;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import util.DomHelpers;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorCode;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;

@RunWith(Parameterized.class)
public class ErrorResultTest {
	private static DocumentBuilder sDocBuilder = DomHelpers.newDocumentBuilder();
	private static RequestHandler<? extends Request> sHandler = new NewSessionRequestHandler();

	private final String mCodeStr;
	private final IfmapErrorCode mCodeType;

	@SuppressWarnings("rawtypes")
	@Parameters
	public static Collection testParams() {
		return Arrays.asList(new Object[][] {
				{"AccessDenied", IfmapErrorCode.AccessDenied},
				{"Failure", IfmapErrorCode.Failure},
				{"InvalidIdentifier", IfmapErrorCode.InvalidIdentifier},
				{"InvalidIdentifierType", IfmapErrorCode.InvalidIdentifierType},
				{"IdentifierTooLong", IfmapErrorCode.IdentifierTooLong},
				{"InvalidMetadata", IfmapErrorCode.InvalidMetadata},
				{"InvalidSchemaVersion", IfmapErrorCode.InvalidSchemaVersion},
				{"InvalidSessionID", IfmapErrorCode.InvalidSessionID},
				{"MetadataTooLong", IfmapErrorCode.MetadataTooLong},
				{"SearchResultsTooBig", IfmapErrorCode.SearchResultsTooBig},
				{"PollResultsTooBig", IfmapErrorCode.PollResultsTooBig},
				{"SystemError", IfmapErrorCode.SystemError}});
	}

	public ErrorResultTest(String errStr, IfmapErrorCode type) {
		mCodeStr = errStr;
		mCodeType = type;
	}

	@Test
	public void fromElementTypeTest() throws UnmarshalException {

		Document doc = sDocBuilder.newDocument();
		Element resp = TestHelpers.makeResponse(doc);
		TestHelpers.appendErrorResult(doc, resp, mCodeStr, "GENERIC", null);

		try {
			sHandler.fromElement(resp);
			fail("IfmapErrorResult expected");
		} catch (IfmapErrorResult e) {
			assertEquals(mCodeType, e.getErrorCode());
		}
	}

}
