package de.fhhannover.inform.trust.ifmapj.messages;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import org.junit.Test;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.MarshalException;
import de.fhhannover.inform.trust.ifmapj.exception.UnmarshalException;

/**
 * Test the handler functionality provided by the {@link Requests} class.
 * 
 * @author aw
 *
 */
public class RequestsTest {

	@Test
	public void testQueryDefaultHandlers() {
		
		assertNotNull(Requests.getHandlerFor(Requests.createNewSessionReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createEndSessionReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createRenewSessionReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createPurgePublisherReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createPublishReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createSubscribeReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createSearchReq()));
		assertNotNull(Requests.getHandlerFor(Requests.createPollReq()));
	}
	
	@Test
	public void testQueryUnknownHandler() {
		assertNull(Requests.getHandlerFor(new Request() {
			
			@Override
			public void setSessionId(String sessionId) {
				// anonymous
			}
			
			@Override
			public String getSessionId() {
				// anonymous
				return null;
			}
		}));
	}

	@Test(expected=NullPointerException.class)
	public void testRegisterNullHandler() {
		Requests.registerRequestHandler(null);
	}

	@Test(expected=RuntimeException.class)
	public void testRegisterDefaultHandlerTwice() {
		Requests.registerRequestHandler(new EndSessionRequestHandler());
	}

	@Test(expected=RuntimeException.class)
	public void testRegisterCustomHandlerTwice() {
		
		class CustomRequest implements Request  {

			@Override
			public void setSessionId(String sessionId) {
				// nothing
			}

			@Override
			public String getSessionId() {
				// nothing
				return null;
			}
		}

		class CustomRequestHandler implements RequestHandler<CustomRequest> {

			@Override
			public Element toElement(Request req, Document doc)
					throws MarshalException {
				return null;
			}

			@Override
			public Result fromElement(Element response)
					throws UnmarshalException, IfmapErrorResult {
				return null;
			}

			@Override
			public Class<CustomRequest> handles() {
				return CustomRequest.class;
			}
		}	
		
		Requests.registerRequestHandler(new CustomRequestHandler());
		Requests.registerRequestHandler(new CustomRequestHandler());
	}

	@Test
	public void testRegisterGoodCustomHandler() {
		
		class CustomRequest implements Request  {

			@Override
			public void setSessionId(String sessionId) {
				// nothing
			}

			@Override
			public String getSessionId() {
				// nothing
				return null;
			}
		}
			
		class CustomRequestHandler implements RequestHandler<CustomRequest> {

			@Override
			public Element toElement(Request req, Document doc)
					throws MarshalException {
				return null;
			}

			@Override
			public Result fromElement(Element response)
					throws UnmarshalException, IfmapErrorResult {
				return null;
			}

			@Override
			public Class<CustomRequest> handles() {
				return CustomRequest.class;
			}
		}	
		
		Requests.registerRequestHandler(new CustomRequestHandler());
		assertNotNull(Requests.getHandlerFor(new CustomRequest()));
	}
	
	@Test(expected=NullPointerException.class)
	public void testRegisterHandlesNullHandler() {
	
		class CustomRequest implements Request  {

			@Override
			public void setSessionId(String sessionId) {
				// nothing
			}

			@Override
			public String getSessionId() {
				// nothing
				return null;
			}
		};

		class CustomRequestHandler implements RequestHandler<CustomRequest> {

			@Override
			public Element toElement(Request req, Document doc)
			throws MarshalException {
				return null;
			}

			@Override
			public Result fromElement(Element response)
			throws UnmarshalException, IfmapErrorResult {
				return null;
			}

			@Override
			public Class<CustomRequest> handles() {
				return null;
			}
		}
		
		// will throw NullPointers, because handles() returns null
		Requests.registerRequestHandler(new CustomRequestHandler());
	}
}
